Thread (42 messages) 42 messages, 3 authors, 2021-07-21

Re: [PATCH 06/14] KVM: arm64: Tolerate re-creating hyp mappings to set ignored bits

From: Fuad Tabba <hidden>
Date: 2021-07-20 10:23:24
Also in: kvmarm, lkml

Hi Quentin,


On Mon, Jul 19, 2021 at 11:47 AM Quentin Perret [off-list ref] wrote:
quoted hunk ↗ jump to hunk
The current hypervisor stage-1 mapping code doesn't allow changing an
existing valid mapping. Relax this condition by allowing changes that
only target ignored bits, as that will soon be needed to annotate shared
pages.

Signed-off-by: Quentin Perret <redacted>
---
 arch/arm64/kvm/hyp/pgtable.c | 18 ++++++++++++++++--
 1 file changed, 16 insertions(+), 2 deletions(-)
diff --git a/arch/arm64/kvm/hyp/pgtable.c b/arch/arm64/kvm/hyp/pgtable.c
index a0ac8c2bc174..34cf67997a82 100644
--- a/arch/arm64/kvm/hyp/pgtable.c
+++ b/arch/arm64/kvm/hyp/pgtable.c
@@ -362,6 +362,17 @@ static int hyp_set_prot_attr(enum kvm_pgtable_prot prot, kvm_pte_t *ptep)
        return 0;
 }

+static bool hyp_pte_needs_update(kvm_pte_t old, kvm_pte_t new)
+{
+       if (old == new)
+               return false;
+
+       if (!kvm_pte_valid(old))
+               return true;
+
+       return !WARN_ON((old ^ new) & ~KVM_PTE_LEAF_ATTR_IGNORED);
Wouldn't this return false if both ignored and non-ignored bits were
different, or is that not possible (judging by the WARN_ON)? If it is,
then it would need an update, wouldn't it?

Thanks,
/fuad

quoted hunk ↗ jump to hunk
+}
+
 static bool hyp_map_walker_try_leaf(u64 addr, u64 end, u32 level,
                                    kvm_pte_t *ptep, struct hyp_map_data *data)
 {
@@ -371,9 +382,12 @@ static bool hyp_map_walker_try_leaf(u64 addr, u64 end, u32 level,
        if (!kvm_block_mapping_supported(addr, end, phys, level))
                return false;

        /* Tolerate KVM recreating the exact same mapping */
        new = kvm_init_valid_leaf_pte(phys, data->attr, level);
-       if (old != new && !WARN_ON(kvm_pte_valid(old)))
+       if (hyp_pte_needs_update(old, new))
                smp_store_release(ptep, new);

        data->phys += granule;
@@ -371,9 +382,12 @@ static bool hyp_map_walker_try_leaf(u64 addr, u64 end, u32 level,
                return false;

-       /* Tolerate KVM recreating the exact same mapping */
+       /*
+        * Tolerate KVM recreating the exact same mapping, or changing ignored
+        * bits.
+        */
        new = kvm_init_valid_leaf_pte(phys, data->attr, level);
--
2.32.0.402.g57bb445576-goog
_______________________________________________
linux-arm-kernel mailing list
linux-arm-kernel@lists.infradead.org
http://lists.infradead.org/mailman/listinfo/linux-arm-kernel
Keyboard shortcuts
hback out one level
jnext message in thread
kprevious message in thread
ldrill in
Escclose help / fold thread tree
?toggle this help